import { IEnumerable } from './enumerable';
import { IEqualityComparer } from './equality-comparer';
import { KeyValuePair } from './key-value-pair';
import { IGrouping } from './grouping';
import { List } from './list';
export interface IDictionary<TKey, TValue> extends IEnumerable<KeyValuePair<TKey, TValue>> {
    Add(key: TKey, value: TValue): void;
    Clear(): void;
    Set(key: TKey, value: TValue): void;
    Get(key: TKey): TValue;
    ContainsKey(key: TKey): boolean;
    Remove(key: TKey): boolean;
    TryGetValue(key: TKey): [boolean, TValue];
}
export declare class Dictionary<TKey, TValue> implements IDictionary<TKey, TValue> {
    private freeList;
    private freeCount;
    private version;
    private count;
    private buckets;
    private entries;
    private comparer;
    constructor(dictionary?: Iterable<KeyValuePair<TKey, TValue>> | Iterable<[TKey, TValue]> | number, comparer?: IEqualityComparer<TKey>);
    private Initialize;
    Add(key: TKey, value: TValue): void;
    Set(key: TKey, value: TValue): void;
    Get(key: TKey): TValue;
    ContainsKey(key: TKey): boolean;
    Clear(): void;
    Remove(key: TKey): boolean;
    TryGetValue(key: TKey): [boolean, TValue];
    private FindEntry;
    private TryInsert;
    private Resize;
    [Symbol.iterator](): IterableIterator<KeyValuePair<TKey, TValue>>;
    All(predicate: (item: KeyValuePair<TKey, TValue>) => boolean): boolean;
    Any(predicate?: (item: KeyValuePair<TKey, TValue>) => boolean): boolean;
    AsEnumerable(): IEnumerable<KeyValuePair<TKey, TValue>>;
    Average(selector?: (item: KeyValuePair<TKey, TValue>) => number): number | null;
    Concat(second: IEnumerable<KeyValuePair<TKey, TValue>>): IEnumerable<KeyValuePair<TKey, TValue>>;
    Contains(value: KeyValuePair<TKey, TValue>, comparer?: IEqualityComparer<KeyValuePair<TKey, TValue>>): boolean;
    Count(predicate?: (item: KeyValuePair<TKey, TValue>) => boolean): number;
    DefaultIfEmpty(defaultValue?: IEnumerable<KeyValuePair<TKey, TValue>>): IEnumerable<KeyValuePair<TKey, TValue>>;
    Distinct(comparer?: IEqualityComparer<KeyValuePair<TKey, TValue>>): IEnumerable<KeyValuePair<TKey, TValue>>;
    ElementAt(index: number): KeyValuePair<TKey, TValue>;
    ElementAtOrDefault(defaultValue: KeyValuePair<TKey, TValue>, index: number): KeyValuePair<TKey, TValue>;
    Except(second: IEnumerable<KeyValuePair<TKey, TValue>>, comparer?: IEqualityComparer<KeyValuePair<TKey, TValue>>): IEnumerable<KeyValuePair<TKey, TValue>>;
    First(predicate?: (item: KeyValuePair<TKey, TValue>) => boolean): KeyValuePair<TKey, TValue>;
    FirstOrDefault(defaultValue: KeyValuePair<TKey, TValue>, predicate?: (item: KeyValuePair<TKey, TValue>) => boolean): KeyValuePair<TKey, TValue>;
    GroupBy<TGroupKey, TElement = KeyValuePair<TKey, TValue>>(keySelector: (item: KeyValuePair<TKey, TValue>) => TGroupKey, elementSelector?: (item: KeyValuePair<TKey, TValue>) => TElement, comparer?: IEqualityComparer<TGroupKey>): IEnumerable<IGrouping<TGroupKey, TElement>>;
    GroupJoin<TInner, TGroupKey, TResult = {
        Outer: KeyValuePair<TKey, TValue>;
        Inners: IEnumerable<TInner>;
    }>(inner: IEnumerable<TInner>, outerKeySelector: (item: KeyValuePair<TKey, TValue>) => TGroupKey, innerKeySelector: (item: TInner) => TGroupKey, resultSelector?: (item: KeyValuePair<TKey, TValue>, inners: IEnumerable<TInner>) => TResult, comparer?: IEqualityComparer<TGroupKey>): IEnumerable<TResult>;
    Intersect(second: IEnumerable<KeyValuePair<TKey, TValue>>, comparer?: IEqualityComparer<KeyValuePair<TKey, TValue>>): IEnumerable<KeyValuePair<TKey, TValue>>;
    Join<TInner, TGroupKey, TResult = {
        Outer: KeyValuePair<TKey, TValue>;
        Inner: TInner;
    }>(inner: IEnumerable<TInner>, outerKeySelector: (item: KeyValuePair<TKey, TValue>) => TGroupKey, innerKeySelector: (item: TInner) => TGroupKey, resultSelector?: (item: KeyValuePair<TKey, TValue>, inners: TInner) => TResult, comparer?: IEqualityComparer<TGroupKey>): IEnumerable<TResult>;
    Last(predicate?: (item: KeyValuePair<TKey, TValue>) => boolean): KeyValuePair<TKey, TValue>;
    LastOrDefault(defaultValue: KeyValuePair<TKey, TValue>, predicate?: (item: KeyValuePair<TKey, TValue>) => boolean): KeyValuePair<TKey, TValue>;
    Max(selector?: (item: KeyValuePair<TKey, TValue>) => number): number | null;
    Min(selector?: (item: KeyValuePair<TKey, TValue>) => number): number | null;
    Reverse(): IEnumerable<KeyValuePair<TKey, TValue>>;
    Select<TResult>(selector: (item: KeyValuePair<TKey, TValue>, index?: number) => TResult): IEnumerable<TResult>;
    SelectMany<TCollection, TResult = TCollection>(collectionSelector: (item: KeyValuePair<TKey, TValue>, index?: number) => IEnumerable<TCollection>, resultSelector?: (item: KeyValuePair<TKey, TValue>, collection: TCollection) => TResult): IEnumerable<TResult>;
    SequenceEqual(second: Iterable<KeyValuePair<TKey, TValue>>, comparer?: IEqualityComparer<KeyValuePair<TKey, TValue>>): boolean;
    Single(predicate?: (item: KeyValuePair<TKey, TValue>) => boolean): KeyValuePair<TKey, TValue>;
    SingleOrDefault(defaultValue: KeyValuePair<TKey, TValue>, predicate?: (item: KeyValuePair<TKey, TValue>) => boolean): KeyValuePair<TKey, TValue>;
    Skip(count: number): IEnumerable<KeyValuePair<TKey, TValue>>;
    SkipWhile(predicate: (item: KeyValuePair<TKey, TValue>, index?: number) => boolean): IEnumerable<KeyValuePair<TKey, TValue>>;
    Sum(selector?: (item: KeyValuePair<TKey, TValue>) => number): number | null;
    Take(count: number): IEnumerable<KeyValuePair<TKey, TValue>>;
    TakeWhile(predicate: (item: KeyValuePair<TKey, TValue>, index?: number) => boolean): IEnumerable<KeyValuePair<TKey, TValue>>;
    ToArray(): Array<KeyValuePair<TKey, TValue>>;
    ToDictionary<TDictionaryKey, TElement = KeyValuePair<TKey, TValue>>(keySelector: (item: KeyValuePair<TKey, TValue>) => TDictionaryKey, elementSelector?: (item: KeyValuePair<TKey, TValue>) => TElement, comparer?: IEqualityComparer<TDictionaryKey>): Dictionary<TDictionaryKey, TElement>;
    ToList(): List<KeyValuePair<TKey, TValue>>;
    Union(second: Iterable<KeyValuePair<TKey, TValue>>, comparer?: IEqualityComparer<KeyValuePair<TKey, TValue>>): IEnumerable<KeyValuePair<TKey, TValue>>;
    Where(predicate: (item: KeyValuePair<TKey, TValue>, index?: number) => boolean): IEnumerable<KeyValuePair<TKey, TValue>>;
}
